/***************************************************************************
 *   Copyright (C) 2006 by André Lindhjem <belgarat@sdf.lonestar.org>,     *
 *                         Kjetil Holien <kjetil.holien@gmail.com>,        *
 *                         Terje Risa <terje.risa@gmail.com> &             *
 *                         Øyvind Nerbråten <oyvind@nerbraten.com>         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************
 *
 * \file callback.h
 * \author André Lindhjem, Kjetil Holien, Terje Risa & Øyvind Nerbråten
 * \date 23.02.2006
 * 
 * Datastructure and methods for controlling it.
 */
 
#ifndef CONTROL_H_
#define CONTROL_H_


// Global variables
struct SmilNode *__smilHead, *__smilTail, *__smilPos;
struct Node *__nodeHead, *__nodeTail, *__nodePos;
struct BookInfo *__bookInfo;
int __ncx;
char *__path;
pthread_t __thread_audio; // Thread for the audio engine

/**
 * the datastructure for infomation about the DTB book 
 */
struct BookInfo
{
	char *titleText; /* the title of the book */
	char *titleAudiofilename; /* the source to the audiofile containing the title */
	char *titleAudioStartPos; /* the time offset for where to start playback in the audio file */
	char *titleAudioStopPos; /* the time offset for where to stop playback in the audio file */
	char *titleImage; /* the source to the book image */
	// TODO: must have a list containing the authors with their respective text,audiofilename, audiostart/stopos and image
};


/**
 * the datastructure for keeping the information of the smilefiles, parsed from the ncc, using a linked list
 * <a> <div> <span>
 */
struct SmilNode
{
	int optional; /* marks optional passages */
	char *id; /* id for the SMIL file */ 
	char *anchor; /* the name of the SMIL file */
	char *header; /* the header of the contens of the smilfile */
	char *fragmentIdentifier; /* identifies a <par> or <text> element in the SMIL file */
	char *textpassage; /* a text corresponding to the smilNode */
	char *audiofilename; /* the filname of the corresponding audio file */
	char *audioStartPos; /* the time offset for where to start playback in the audio file */
	char *audioStopPos; /* the time offset for where to stop playback in the audio file */
	char *image; /* the source to the corresponding image */
	struct SmilNode *next; /* a pointer to the next smilnode in the linked list */
	struct SmilNode *prev;	/* a pointer to the previous smilnode in the linked list */
};

 
/**
 * The datastructure for a passage within the book. Holds the name and audio_play times of the corresponding
 * audio file and the text for the passage. Passages is stored in a linked list.
 */ 
struct Node
{ 
	//int skippable; /* true if the node is auto skippable (normally false) */
	char *audiofilename; /* the filname of the corresponding audio file */
	char *audioStartPos; /* the time offset for where to start playback in the audio file */
	char *audioStopPos; /* the time offset for where to stop playback in the audio file */
	char *textfilename; /* the filename of the corresponding text file */
	char *fragmentidentifier; /* identificator for the corresponding passage in the text file */
	char *textPassage; /* the corresponding text passage */
	char *image; /* the source to the corresponding image */
	struct Node *next; /* a pointer to the next node in the lined list */
	struct Node *prev; /* a pointer to the previous node int the linked list */
};


//  Function declarations
void parseInit ();
void parseTerminate ();
char* removewhitespaces(char* string);
char* getTime(char* time);
struct Node* addNewNode ();
struct SmilNode* addNewSmilNode ();
int seek (int type, int book);
int parse(char* filename);
void start_audio (void (*cb)(const void *)); //starts and stops the audio-engine as a thread.
void stop_audio (); //starts and stops the audio-engine as a thread.


#endif /*CONTROL_H_*/
